﻿// Fill out your copyright notice in the Description page of Project Settings.


#include "LMGameInstance.h"

#include "LMSaveGame.h"
#include "Kismet/GameplayStatics.h"

void ULMGameInstance::SaveSettings()
{
	const bool bSuccess = UGameplayStatics::SaveGameToSlot(SaveGame, SaveGameSlot, UserIndex);
	if (!bSuccess)
	{
		UE_LOG(LogTemp, Error, TEXT("Failed to save game settings."));
	}
	else
	{
		UE_LOG(LogTemp, Display, TEXT("Saved game settings"));
	}
	SettingsSaved(SaveGameSlot, UserIndex, bSuccess);
}

void ULMGameInstance::AsyncSaveSettings()
{
	FAsyncSaveGameToSlotDelegate SavedDelegate;
	SavedDelegate.BindUObject(this, &ULMGameInstance::SettingsSaved);
	
	UGameplayStatics::AsyncSaveGameToSlot(SaveGame, SaveGameSlot, UserIndex, SavedDelegate);
}

void ULMGameInstance::SettingsSaved_Implementation(const FString& SlotName, const int32 Index, bool bSuccess)
{
	if (!bSuccess)
	{
		UE_LOG(LogTemp, Error, TEXT("Failed to save game settings %s with user %i."), *SlotName, Index);
	}
	else
	{
		UE_LOG(LogTemp, Display, TEXT("Successfully saved game settings %s for user %i"), *SlotName, Index);
	}
}

void ULMGameInstance::LoadSettings()
{
	if (ULMSaveGame* UnrealSaveGame = Cast<ULMSaveGame>(UGameplayStatics::LoadGameFromSlot(SaveGameSlot, UserIndex)))
	{
		SaveGame = UnrealSaveGame;
		UE_LOG(LogTemp, Display, TEXT("Loaded game settings !"));
	}
	else
	{
		if (!SaveGame)
		{
			SaveGame = CastChecked<ULMSaveGame>(UGameplayStatics::CreateSaveGameObject(ULMSaveGame::StaticClass()));
		}
		
		if (!UGameplayStatics::SaveGameToSlot(SaveGame, SaveGameSlot, UserIndex))
		{
			UE_LOG(LogTemp, Error, TEXT("Failed to create settings save."))
		}
		else
		{
			UE_LOG(LogTemp, Display, TEXT("Created game settings."));
		}
	}
}
