// Fill out your copyright notice in the Description page of Project Settings.


#include "Player/LMPlayerController.h"
#include "EnhancedInputSubsystems.h"
#include "EnhancedInputComponent.h"
#include "LMUserSettings.h"
#include "GameFramework/Character.h"
#include "Kismet/GameplayStatics.h"
#include "Player/LMPlayer.h"
#include "Weapon/LMWeaponManager.h"

void ALMPlayerController::BeginPlay()
{
	Super::BeginPlay();
	EnhancedInputSubsystem = GetLocalPlayer()->GetSubsystem<UEnhancedInputLocalPlayerSubsystem>();
	EnhancedInputSubsystem->AddMappingContext(DefaultMappingContext, 0);
	LegumixPlayer = Cast<ALMPlayer>(GetPawn());

	LegumixPlayer->OnRequestedUnpause.AddUniqueDynamic(this, &ALMPlayerController::TogglePause);
	ULMUserSettings::GetLegumixUserSettings()->OnSettingsChanged.AddUniqueDynamic(this, &ALMPlayerController::OnSettingsChanged);

	OnSettingsChanged();
}

void ALMPlayerController::SetupInputComponent()
{
	Super::SetupInputComponent();
	if(UEnhancedInputComponent* Input = Cast<UEnhancedInputComponent>(InputComponent))
	{
		Input->BindAction(MoveAction, ETriggerEvent::Triggered, this, &ALMPlayerController::Move);
		Input->BindAction(JumpAction, ETriggerEvent::Triggered, this, &ALMPlayerController::Jump);
		Input->BindAction(LookAction, ETriggerEvent::Triggered, this, &ALMPlayerController::Look);
		Input->BindAction(FireAction, ETriggerEvent::Triggered, this, &ALMPlayerController::Fire);
		Input->BindAction(ReloadAction, ETriggerEvent::Triggered, this, &ALMPlayerController::Reload);
		Input->BindAction(ScrollAction, ETriggerEvent::Triggered, this, &ALMPlayerController::Scroll);
		Input->BindAction(PauseAction, ETriggerEvent::Triggered, this, &ALMPlayerController::TogglePause);
	}
}

void ALMPlayerController::Move(const FInputActionValue& InputValue)
{
	const FVector2d VectorDirection = InputValue.Get<FVector2d>();
	GetPawn()->AddMovementInput(GetPawn()->GetActorForwardVector(), VectorDirection.X);
	GetPawn()->AddMovementInput(GetPawn()->GetActorRightVector(), VectorDirection.Y);
}

void ALMPlayerController::Jump(const FInputActionValue& InputValue)
{
	LegumixPlayer->Jump();
}

void ALMPlayerController::Look(const FInputActionValue& InputValue)
{
	const FVector2d VectorDirection = InputValue.Get<FVector2d>();
	
	AddYawInput(VectorDirection.X * Sensitivity);
	AddPitchInput(VectorDirection.Y * Sensitivity);
}

void ALMPlayerController::Fire(const FInputActionValue& InputValue)
{
	LegumixPlayer->GetWeaponManager()->Fire();
}

void ALMPlayerController::Reload(const FInputActionValue& InputValue)
{
	LegumixPlayer->GetWeaponManager()->Reload();
}

void ALMPlayerController::Scroll(const FInputActionValue& InputValue)
{
	const float ScrollAmount = InputValue.Get<float>();
	LegumixPlayer->GetWeaponManager()->SwitchWeapon(ScrollAmount);
}

void ALMPlayerController::PauseInput(const FInputActionValue& InputValue)
{
	TogglePause();
}

void ALMPlayerController::TogglePause()
{
	const bool bIsPaused = UGameplayStatics::IsGamePaused(GetWorld());
	UGameplayStatics::SetGamePaused(GetWorld(), !bIsPaused);
	SetShowMouseCursor(!bIsPaused);

	LegumixPlayer->OnPause(!bIsPaused);

	if (!bIsPaused)
	{
		const FInputModeGameOnly InputMode;
		SetInputMode(InputMode);
	}
}

void ALMPlayerController::OnSettingsChanged()
{
	Sensitivity = ULMUserSettings::GetLegumixUserSettings()->MouseSensitivity;
	int const CameraFov = ULMUserSettings::GetLegumixUserSettings()->FieldOfView;
	LegumixPlayer->GetCamera()->SetFieldOfView(CameraFov);
}
