﻿// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Engine/GameInstance.h"
#include "LMGameInstance.generated.h"

class ULMSaveGame;
/**
 * 
 */
UCLASS()
class LEGUMEMIX_API ULMGameInstance : public UGameInstance
{
	GENERATED_BODY()

public:
	/**
	 * Saves the current settings.
	 */
	UFUNCTION(BlueprintCallable, Category=Legumix)
	void SaveSettings();
	/**
	 * Saves the current settings asynchronously.
	 */
	UFUNCTION(BlueprintCallable, Category=Legumix)
	void AsyncSaveSettings();

	/**
	 * Callback called when a save is finalized.
	 * @param SlotName The slot that was saved.
	 * @param Index The user index assigned to the slot.
	 * @param bSuccess Whether the save succeeded or not.
	 */
	UFUNCTION(BlueprintNativeEvent, Category=Legumix)
	void SettingsSaved(const FString& SlotName, int32 Index, bool bSuccess);

	/**
	 * Load the save settings.
	 */
	UFUNCTION(BlueprintCallable, Category=Legumix)
	void LoadSettings();

private:
	/**
	 * The save game to use.
	 */
	UPROPERTY(BlueprintReadWrite, EditAnywhere, Category=Legumix, meta=(AllowPrivateAccess=true))
	TObjectPtr<ULMSaveGame> SaveGame;

	/**
	 * The save game slot to use for the save.
	 */
	UPROPERTY(BlueprintReadWrite, EditAnywhere, Category=Legumix, meta=(AllowPrivateAccess=true))
	FString SaveGameSlot;

	int UserIndex = 0;
};
