﻿// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "LMBulletTracer.generated.h"

class UNiagaraComponent;

UCLASS(Blueprintable)
class LEGUMEMIX_API ALMBulletTracer : public AActor
{
	GENERATED_BODY()

public:
	ALMBulletTracer();
	void BeginPlay() override;
	virtual void Tick(float DeltaTime) override;

	void Initialize(const FVector& Vector, const FRotator& Direction, USceneComponent* Origin);
public:
	UFUNCTION(BlueprintCallable, Category=Legumix)
	void DestinationReached();
	
	/** Called automatically when the bullet reaches it's destination: it hit something.
	 * The Actor is then automatically destroyed after *TrailLifetime* seconds.
	 */
	UFUNCTION(BlueprintImplementableEvent)
	void OnDestinationReached();

	/** Called automatically when the bullet has been given a direction.
	 * @param Lifetime The number of seconds before this object reaches it's destination.
	 */
	UFUNCTION(BlueprintCallable, BlueprintImplementableEvent, Category=Legumix)
	void OnIntialized(float Lifetime);

private:
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category=Legumix, meta=(AllowPrivateAccess=true))
	float Speed = 10000.f;

	/** The lifetime of a single particle emitted by the niagara system. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category=Legumix, meta=(AllowPrivateAccess=true))
	float TrailLifeTime = 2.f;

	UPROPERTY(VisibleAnywhere, BlueprintReadWrite, Category=Legumix, meta=(AllowPrivateAccess=true))
	TObjectPtr<UNiagaraComponent> TrailFx;

	UPROPERTY(BlueprintReadWrite, Category=Legumix, meta=(AllowPrivateAccess=true))
	bool bCanMove;

	UPROPERTY(BlueprintReadWrite, Category=Legumix, meta=(AllowPrivateAccess=true))
	TObjectPtr<USceneComponent> TrailOrigin;
	
private:
	FTimerHandle TimerHandle;
};
