﻿// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Ammo/LMAmmo.h"
#include "LMWeaponState.h"
#include "GameFramework/Actor.h"
#include "LMWeaponBase.generated.h"


class UFMODEvent;
class ALMPlayer;

UCLASS()
class LEGUMEMIX_API ALMWeaponBase : public AActor
{
	GENERATED_BODY()

public:
	ALMWeaponBase();
	virtual void BeginPlay() override;
	void Setup(USkeletalMeshComponent* Mesh, AActor* CharOwner);
	
public:
	UFUNCTION(BlueprintNativeEvent, BlueprintCallable, Category=Legumix)
	void OnEquip();
	virtual void OnEquip_Implementation();
	
	UFUNCTION(BlueprintCallable)
	virtual bool Reload();
	
	UFUNCTION(BlueprintCallable)
	virtual void PrimaryFire();

	UFUNCTION(BlueprintCallable)
	void PlaySound(USoundWave* Sound, bool Replacing = false);
	UFUNCTION(BlueprintCallable)
	void PlayEvent(UFMODEvent* Event) const;

	UFUNCTION(BlueprintCallable)
	void PlayAnimation(UAnimMontage* Animation);

	/** The socket to attach the weapon to.
	 * @return The socket.
	 */
	UFUNCTION(BlueprintCallable)
	FName GetAttachmentSocketName() const { return WeaponSocket; }
	
	UFUNCTION(BlueprintCallable, BlueprintNativeEvent)
	void OnDefaultReload();

	UFUNCTION(BlueprintImplementableEvent, BlueprintCallable)
	void OnFire();

protected:
	UFUNCTION(BlueprintCallable)
	void AllowRefire();
	
private:
	bool DefaultReload();

public:
	/** The current state of the weapon. */
	UPROPERTY(VisibleInstanceOnly, BlueprintReadWrite, Category="Legumix|Weapon")
	EWeaponState State;
	
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category=Legumix, meta=(AllowPrivateAccess=true))
	TObjectPtr<USceneComponent> TracerOrigin;

protected: /* Weapon Data */
	FTimerHandle FireTimer;
	FTimerHandle ReloadTimer;
	
	/** The sound to play when firing. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite,Category="Legumix|Sounds")
	TObjectPtr<UFMODEvent> FireEvent;

	UPROPERTY(EditAnywhere, BlueprintReadWrite,Category="Legumix|Sounds")
	TObjectPtr<UFMODEvent> ReloadSound;
	
	UPROPERTY(EditAnywhere, BlueprintReadWrite,Category="Legumix|Sounds")
	TObjectPtr<USoundWave> DryFireSound;
	
	UPROPERTY(EditAnywhere, BlueprintReadWrite,Category="Legumix|Sounds")
	TObjectPtr<USoundWave> EquipSound;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Animations", meta=(AllowPrivateAccess=true))
	TObjectPtr<UAnimMontage> PrimaryFireAnimation;

	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Legumix|Animations", meta=(AllowPrivateAccess=true))
	TObjectPtr<UAnimMontage> ReloadAnimation;

	/** The number of seconds before being able to fire again. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	float RefireDelay = 0.5f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	float ReloadDelay = 0.5f;

	/** The flat damage number of one bullet. */
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	float Damage = 10.f;

	/** The max angle deviation of a bullet from the original ray. In degrees.*/
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true, UIMin=0, UIMax=180))
	float WeaponSpread = 10;

	/** The max distance (cm) between the origin and the target before doing minimal damage. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	float MaxDistance = 100000.f;

	/** The damage falloff distance in a 0-1 scale. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	TObjectPtr<UCurveFloat> DamageFalloff;
	
	/** The type of ammo used by this weapon. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	EAmmoType AmmoType;

	/** The current number of ammo in the clip. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	int ClipAmmo;
	
	/** The maximum amount of ammo in a clip. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	int MaxClip;
	
	UPROPERTY(VisibleInstanceOnly, BlueprintReadWrite, Category=Legumix)
	TObjectPtr<ALMPlayer> Player;
	
private:
	/** An optional socket to attach the weapon to. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category=Legumix, meta=(AllowPrivateAccess=true))
	FName WeaponSocket;

private: /* Components */
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category=Legumix, meta=(AllowPrivateAccess=true))
	TObjectPtr<UAudioComponent> AudioComponent;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category=Legumix, meta=(AllowPrivateAccess=true))
	TObjectPtr<USkeletalMeshComponent> WeaponMesh;
};

